@@ -4,11 +4,9 @@
 
 #include <linux/input-event-codes.h>
 #include "retro-cairo-display.h"
+#include "retro-controller-codes.h"
 #include "retro-core-view-controller.h"
-#include "retro-joypad-id.h"
-#include "retro-mouse-id.h"
 #include "retro-pa-player.h"
-#include "retro-pointer-id.h"
 
 static guint16 DEFAULT_KEY_JOYPAD_BUTTON_MAPPING[RETRO_JOYPAD_ID_COUNT] = {
   KEY_S,
@@ -620,34 +618,60 @@ retro_core_view_as_controller (RetroCoreView       *self,
   return RETRO_CONTROLLER (retro_core_view_controller_new (self, controller_type));
 }
 
+/**
+ * retro_core_view_set_as_default_controller:
+ * @self: a #RetroCoreView
+ * @core: (nullable): a #RetroCore
+ *
+ * Sets the @self as the default controllers of @core.
+ */
+void
+retro_core_view_set_as_default_controller (RetroCoreView *self,
+                                           RetroCore     *core)
+{
+  RetroControllerType type;
+  RetroController *controller;
+
+  g_return_if_fail (RETRO_IS_CORE_VIEW (self));
+  g_return_if_fail (RETRO_IS_CORE (core));
+
+  for (type = RETRO_CONTROLLER_TYPE_NONE;
+       type < RETRO_CONTROLLER_TYPE_COUNT;
+       type++) {
+    controller = retro_core_view_as_controller (self, type);
+    retro_core_set_default_controller (core, type, controller);
+    g_object_unref (controller);
+  }
+}
+
 /**
  * retro_core_view_get_input_state:
  * @self: a #RetroCoreView
- * @controller_type: a #RetroControllerType to query @self
- * @index: an input index to interpret depending on @device
- * @id: an input id to interpret depending on @device
+ * @input: a #RetroInput to query @self
  *
  * Gets the state of an input of @self.
  *
  * Returns: the input's state
  */
 gint16
-retro_core_view_get_input_state (RetroCoreView       *self,
-                                 RetroControllerType  controller_type,
-                                 guint                index,
-                                 guint                id)
+retro_core_view_get_input_state (RetroCoreView *self,
+                                 RetroInput    *input)
 {
+  guint id;
   gint16 result;
 
   g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), 0);
 
-  switch (controller_type) {
+  switch (retro_input_get_controller_type (input)) {
   case RETRO_CONTROLLER_TYPE_JOYPAD:
-    if (id >= RETRO_JOYPAD_ID_COUNT)
+    if (!retro_input_get_joypad (input, &id))
       return 0;
 
     return retro_core_view_get_joypad_button_state (self, id) ? G_MAXINT16 : 0;
   case RETRO_CONTROLLER_TYPE_MOUSE:
+    if (!retro_input_get_mouse (input, &id))
+      return 0;
+
     switch (id) {
     case RETRO_MOUSE_ID_X:
       result = (gint16) self->mouse_x_delta;
@@ -667,6 +691,9 @@ retro_core_view_get_input_state (RetroCoreView       *self,
       return 0;
     }
   case RETRO_CONTROLLER_TYPE_POINTER:
+    if (!retro_input_get_pointer (input, &id))
+      return 0;
+
     switch (id) {
     case RETRO_POINTER_ID_X:
       return axis_to_retro_axis (self->pointer_x);
